(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 09-Feb-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: Blobs.cpp
//
// Desc: A pixel shader effect to mimic metaball physics in image space.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit BlobsUnit;

interface

uses
  Windows, Messages,
  Direct3D9, D3DX9, dxerr9,
  D3DApp, D3DFont, D3DUtil, DXUtil,
  Resource;

// C++Builder compile hack
{$NOINCLUDE Resource}
{$HPPEMIT '#include "Resource.hpp"'}

{.$DEFINE DEBUG_VS}   // Uncomment this line to debug vertex shaders
{.$DEFINE DEBUG_PS}   // Uncomment this line to debug pixel shaders


//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------
const
  NUM_BLOBS           = 5;
  FIELD_OF_VIEW       = (70.0/90.0)*({D3DX_PI}3.141592654/2.0);

  GAUSSIAN_TEXSIZE    = 64;
  GAUSSIAN_HEIGHT     = 1;
  GAUSSIAN_DEVIATION  = 0.125;


type
  // Vertex format for blob billboards
  PPointVertex = ^TPointVertex;
  TPointVertex = packed record
    pos: TD3DXVector3;
    size: Single;
    color: TD3DXVector3;
  end;

const
  TPointVertex_FVF = D3DFVF_XYZ or D3DFVF_PSIZE or D3DFVF_DIFFUSE;

type
  // Vertex format for screen space work
  PScreenVertex = ^TScreenVertex;
  TScreenVertex = packed record
    pos: TD3DXVector4;
    t0: TD3DXVector2;
    t1: TD3DXVector2;
    t2: TD3DXVector2;
    t3: TD3DXVector3;
  end;
  PScreenVertexArray = ^TScreenVertexArray;
  TScreenVertexArray = array[0..MaxInt div SizeOf(TScreenVertex)-1] of TScreenVertex;
const
  TScreenVertex_FVF = D3DFVF_XYZRHW or D3DFVF_TEX4 or D3DFVF_TEXTUREFORMAT2{==0};
                               { or D3DFVF_TEXCOORDSIZE2(0) or
                                 or  D3DFVF_TEXCOORDSIZE2(1)
                                 or  D3DFVF_TEXCOORDSIZE2(2)
                                 or  D3DFVF_TEXCOORDSIZE3(3); }


type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Application class. The base class (CD3DApplication) provides the
  //       generic functionality needed in all Direct3D samples. CMyD3DApplication
  //       adds functionality specific to this sample program.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  private
    m_pBlobVB: IDirect3DVertexBuffer9;     // Vertex buffer for blob billboards

    m_BlobPoints: array[0..NUM_BLOBS-1] of TPointVertex; // Position, size, and color states

    m_pFont: ID3DXFont;                    // Font for drawing text

    m_matWorld: TD3DXMatrixA16;            // World matrix
    m_matView: TD3DXMatrixA16;             // View matrix
    m_matProjection: TD3DXMatrixA16;       // Projection matrix
    m_matWorldView: TD3DXMatrixA16;        // World-view concatenation

    m_pTexGBuffer: array[0..3] of IDirect3DTexture9;   // Buffer textures for blending effect
    m_pTexScratch: IDirect3DTexture9;      // Scratch texture
    m_pTexBlob: IDirect3DTexture9;         // Blob texture

    m_pEnvMap: IDirect3DCubeTexture9;      // Environment map
    m_Camera: CModelViewerCamera;          // Arcball camera
    m_pEffect: ID3DXEffect;                // Effect
    m_fAnimationTime: Single;              // Animation clock

    m_bShowHelp: Boolean;                  // Toggle on-screen help

  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;

    function RenderFullScreenQuad(fDepth: Single): HRESULT;
    function FillBlobVB: HRESULT;
    procedure RenderText;
    function GenerateGaussianTexture: HRESULT;

  public
    constructor Create;
    function MsgProc(hWnd: HWND; uMsg: LongWord; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;



implementation

uses SysUtils, D3DEnumeration;

const
  UnitName = 'BlobUnit.pas';

//-----------------------------------------------------------------------------
// Name: BlobSortCB()
// Desc: Callback function for sorting blobs in ascending z order
//-----------------------------------------------------------------------------
function BlobSortCB(const a, b: Pointer): Integer; 
begin
  if PPointVertex(a).pos.z < PPointVertex(b).pos.z then Result:= -1
  else
    if PPointVertex(a).pos.z > PPointVertex(b).pos.z then Result:= 1
      else Result:= 0;
end;


{ CMyD3DApplication }

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.
//       The application window has not yet been created and no Direct3D device
//       has been created, so any initialization that depends on a window or
//       Direct3D should be deferred to a later stage.
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
begin
  inherited Create;
  m_Camera:= CModelViewerCamera.Create;

  m_strWindowTitle := 'Blobs';
  m_d3dEnumeration.AppUsesDepthBuffer := True;
  m_dwCreationWidth  := 640;
  m_dwCreationHeight := 480;
  m_pFont := nil;
  m_bShowHelp := True;
  m_pEffect := nil;
  m_pTexScratch := nil;
  m_pTexBlob := nil;
  m_pEnvMap := nil;
  m_fAnimationTime := 0.0;
  m_pEffect := nil;
  m_pBlobVB := nil;

  ZeroMemory(@m_pTexGBuffer, SizeOf(m_pTexGBuffer));
  ZeroMemory(@m_BlobPoints, SizeOf(m_BlobPoints));
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
var
  i: Integer;
begin
  // Set initial blob states
  for i:= 0 to NUM_BLOBS - 1 do
  begin
    m_BlobPoints[i].pos := D3DXVector3Zero;
    m_BlobPoints[i].size := 1.0;
  end;

  m_BlobPoints[0].color := D3DXVector3(0.3, 0.0, 0.0);
  m_BlobPoints[1].color := D3DXVector3(0.0, 0.3, 0.0);
  m_BlobPoints[2].color := D3DXVector3(0.0, 0.0, 0.3);
  m_BlobPoints[3].color := D3DXVector3(0.3, 0.3, 0.0);
  m_BlobPoints[4].color := D3DXVector3(0.0, 0.3, 0.3);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  hDC: Windows.HDC;
  nHeight: Integer;
  dwShaderFlags: DWORD;
  pBufferErrors: ID3DXBuffer;
  str: array[0..MAX_PATH-1] of Char;
begin
  hDC := GetDC(0);
  nHeight := -MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72);
  ReleaseDC(0, hDC);
  // Create a font for statistics and help output
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, False, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
                           'Arial', m_pFont);
  if FAILED(Result) then Exit;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug
  // experience in the shader debugger.  It enables source level debugging, prevents instruction
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next
  // higher available software target, which ensures that the unoptimized shaders do not exceed
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders
  // will be unoptimized and forced into software.  See the DirectX documentation for more information
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  // Create the effect
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'Blobs.fx');
  Result := D3DXCreateEffectFromFile(m_pd3dDevice, str, nil, nil, dwShaderFlags, nil, m_pEffect, @pBufferErrors);
  if FAILED(Result) then Exit;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  i: LongWord;
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
begin
  if Assigned(m_pFont) then m_pFont.OnResetDevice;
  if Assigned(m_pEffect) then m_pEffect.OnResetDevice;

  Result:= GenerateGaussianTexture;
  if FAILED(Result) then Exit;
   
  // Create the blob vertex buffer
  Result := m_pd3dDevice.CreateVertexBuffer(NUM_BLOBS * 6 * SizeOf(TScreenVertex),
                                            D3DUSAGE_WRITEONLY or D3DUSAGE_DYNAMIC,
                                            TScreenVertex_FVF, D3DPOOL_DEFAULT,
                                            m_pBlobVB, nil);
  if FAILED(Result) then Exit;

  // Create buffer textures
  for i:= 0 to 3 do
  begin
    Result:= m_pd3dDevice.CreateTexture(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height, 1,
                                        D3DUSAGE_RENDERTARGET,
                                        D3DFMT_A16B16G16R16F,
                                        D3DPOOL_DEFAULT, m_pTexGBuffer[i], nil);
    if FAILED(Result) then Exit;
  end;


  // Create the blank texture
  Result := m_pd3dDevice.CreateTexture(1, 1, 1,
                                       D3DUSAGE_RENDERTARGET,
                                       D3DFMT_A16B16G16R16F,
                                       D3DPOOL_DEFAULT,
                                       m_pTexScratch, nil);

  if FAILED(Result) then Exit;

  // Set the camera parameters
  vEyePt    := D3DXVector3(0.0, -2.0, -5.0);
  vLookatPt := D3DXVector3(0.0, 0.0, 0.0);
  m_Camera.SetViewParams(vEyePt, vLookatPt);
  m_Camera.SetProjParams(FIELD_OF_VIEW, m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height, 1.0, 100.0);
  m_Camera.SetRadius(5.0, 2.0, 20.0);
  m_Camera.SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);
  m_Camera.SetInvertPitch(True);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: GenerateGaussianTexture()
// Desc: Generate a texture to store gaussian distribution results
//-----------------------------------------------------------------------------
function CMyD3DApplication.GenerateGaussianTexture: HRESULT;
var
  pBlobTemp: IDirect3DSurface9;
  pBlobNew: IDirect3DSurface9;
  texTemp: IDirect3DTexture9;
  str: array[0..MAX_PATH-1] of Char;
  Rect: TD3DLockedRect;
  u, v: Integer;
  dx, dy, I: Single;
  pBits: PSingle;
begin
  // Create a temporary texture
  Result := m_pd3dDevice.CreateTexture(GAUSSIAN_TEXSIZE, GAUSSIAN_TEXSIZE, 1,
                                       D3DUSAGE_DYNAMIC, D3DFMT_R32F,
                                       D3DPOOL_DEFAULT, texTemp, nil);
  if FAILED(Result) then Exit;

  // Create the gaussian texture
  Result := m_pd3dDevice.CreateTexture(GAUSSIAN_TEXSIZE, GAUSSIAN_TEXSIZE, 1,
                                       D3DUSAGE_DYNAMIC, D3DFMT_R16F,
                                       D3DPOOL_DEFAULT, m_pTexBlob, nil);
  if FAILED(Result) then Exit;


  // Create the environment map
  DXUtil_FindMediaFileCb( str, SizeOf(str), 'lobbycube.dds');
  Result := D3DXCreateCubeTextureFromFile(m_pd3dDevice, str, m_pEnvMap);
  if FAILED(Result) then Exit;


  // Fill in the gaussian texture data
  Result := texTemp.LockRect(0, Rect, nil, 0);
  if FAILED(Result) then Exit;

  for v:= 0 to GAUSSIAN_TEXSIZE - 1 do
  begin
    pBits := PSingle(Integer(Rect.pBits)+v*Rect.Pitch);

    for u:= 0 to GAUSSIAN_TEXSIZE - 1 do
    begin
      dx := 2.0*u/GAUSSIAN_TEXSIZE-1.0;
      dy := 2.0*v/GAUSSIAN_TEXSIZE-1.0;
      I := GAUSSIAN_HEIGHT * Exp(-(dx*dx+dy*dy)/GAUSSIAN_DEVIATION);

      pBits^:= I; Inc(pBits); // intensity
    end;
  end;

  texTemp.UnlockRect(0);

  // Copy the temporary surface to the stored gaussian texture
  texTemp.GetSurfaceLevel(0, pBlobTemp);
  m_pTexBlob.GetSurfaceLevel(0, pBlobNew);

  Result := D3DXLoadSurfaceFromSurface(pBlobNew, nil, nil, pBlobTemp, nil, nil, D3DX_FILTER_NONE, 0);
  if FAILED(Result) then Exit;

  Result := S_OK;

{ // Delphi automatically does this
LCleanReturn:
  SAFE_RELEASE(pBlobTemp);
  SAFE_RELEASE(pBlobNew);
  SAFE_RELEASE(texTemp); }
end;


//-----------------------------------------------------------------------------
// Name: FillBlobVB()
// Desc: Fill the vertex buffer for the blob objects
//-----------------------------------------------------------------------------
function CMyD3DApplication.FillBlobVB: HRESULT;
var
  i, j: LongWord; // Loop variable
  pBlobVertex: PScreenVertexArray;
  blobPos: array[0..NUM_BLOBS-1] of TPointVertex;
  blobPosCamera: TD3DXVector4;
  posCount: Integer;

  blobScreenPos: TD3DXVector4;
  billOfs: TD3DXVector4;
  billOfsScreen: TD3DXVector4;
  zOfs: Single;
const
  vTexCoords: array[0..5] of TD3DXVector2 =
   (
    (x:0.0; y:0.0),
    (x:0.0; y:1.0),
    (x:1.0; y:1.0),
    (x:1.0; y:1.0),
    (x:1.0; y:0.0),
    (x:0.0; y:0.0)
  );
var
  vPosOffset: array[0..5] of TD3DXVector4;
begin
  Result := m_pBlobVB.Lock(0, 0, Pointer(pBlobVertex), D3DLOCK_DISCARD);
  if FAILED(Result) then Exit;

  for i:= 0 to NUM_BLOBS - 1 do
  begin
    //transform point to camera space
    D3DXVec3Transform(blobPosCamera, m_BlobPoints[i].pos, m_matWorldView);

    blobPos[i] := m_BlobPoints[i];
    blobPos[i].pos.x := blobPosCamera.x;
    blobPos[i].pos.y := blobPosCamera.y;
    blobPos[i].pos.z := blobPosCamera.z;
  end;

  // Sort by ascending z
  QSort(@blobPos, NUM_BLOBS, SizeOf(TPointVertex), BlobSortCB);

  posCount:= 0;
  for i:= 0 to NUM_BLOBS - 1 do
  begin
    // For calculating billboarding
    billOfs:= D3DXVector4(blobPos[i].size, blobPos[i].size, blobPos[i].pos.z, 1);

    // Transform to screenspace
    D3DXVec3Transform(blobScreenPos, blobPos[i].pos, m_matProjection);
    D3DXVec4Transform(billOfsScreen, billOfs, m_matProjection);

    // Project
    zOfs := blobScreenPos.z;
    D3DXVec4Scale(blobScreenPos, blobScreenPos, 1.0/blobScreenPos.w);
    D3DXVec4Scale(billOfsScreen, billOfsScreen, 1.0/billOfsScreen.w);

    vPosOffset[0]:= D3DXVector4(-billOfsScreen.x, -billOfsScreen.y, 0.0, 0.0);
    vPosOffset[1]:= D3DXVector4(-billOfsScreen.x,  billOfsScreen.y, 0.0, 0.0);
    vPosOffset[2]:= D3DXVector4( billOfsScreen.x,  billOfsScreen.y, 0.0, 0.0);
    vPosOffset[3]:= D3DXVector4( billOfsScreen.x,  billOfsScreen.y, 0.0, 0.0);
    vPosOffset[4]:= D3DXVector4( billOfsScreen.x, -billOfsScreen.y, 0.0, 0.0);
    vPosOffset[5]:= D3DXVector4(-billOfsScreen.x, -billOfsScreen.y, 0.0, 0.0);

    // Set constants across quad
    for j:= 0 to 5 do 
    begin
      // Scale to pixels
      D3DXVec4Add(pBlobVertex[posCount].pos, blobScreenPos, vPosOffset[j]);

      with pBlobVertex[posCount] do
      begin
        pos.x := pos.x * m_d3dsdBackBuffer.Width;
        pos.y := pos.y * m_d3dsdBackBuffer.Height;
        pos.x := pos.x + 0.5 * m_d3dsdBackBuffer.Width;
        pos.y := pos.y + 0.5 * m_d3dsdBackBuffer.Height;

        t0 := vTexCoords[j];
        t1 := D3DXVector2((0.5 + pos.x)*(1.0/m_d3dsdBackBuffer.Width),
                          (0.5 + pos.y)*(1.0/m_d3dsdBackBuffer.Height));
        t2.x := zOfs;
        t2.y := blobPos[i].size;
        t3 := blobPos[i].color;
      end;

      Inc(posCount);
    end;
  end;
  m_pBlobVB.Unlock;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
var
  i: LongWord;
begin
  SAFE_RELEASE(m_pBlobVB);
  SAFE_RELEASE(m_pTexScratch);
  SAFE_RELEASE(m_pTexBlob);
  SAFE_RELEASE(m_pEnvMap);

  for i:=0 to 3 do SAFE_RELEASE(m_pTexGBuffer[i]);

  if Assigned(m_pFont) then m_pFont.OnLostDevice;
  if Assigned(m_pEffect) then  m_pEffect.OnLostDevice;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  SAFE_RELEASE(m_pFont);
  SAFE_RELEASE(m_pEffect);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: LongWord;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
begin
  // Pass mouse messages to the ArcBall so it can build internal matrices
  m_Camera.HandleMessages(hWnd, uMsg, wParam, lParam);

  case uMsg of
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDM_TOGGLEHELP: m_bShowHelp := not m_bShowHelp;
      end;
    end;
  end;

  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
var
  matWorldViewProjection: TD3DXMatrixA16;
  fAnimationDelta: Single;
  pos: Single;
begin
  // Update the camera
  m_Camera.FrameMove(m_fElapsedTime);

  // Get world/view/projection matrices from the camera class
  m_matWorld      := m_Camera.GetWorldMatrix^;
  m_matView       := m_Camera.GetViewMatrix^;
  m_matProjection := m_Camera.GetProjMatrix^;

  // Calc matricies
  D3DXMatrixMultiply(m_matWorldView, m_matWorld, m_matView);
  D3DXMatrixMultiply(matWorldViewProjection, m_matWorldView, m_matProjection);

  // Pass the composed transformation matrix to the effect
  m_pEffect.SetMatrix('g_mWorldViewProjection', matWorldViewProjection);
    
  // Pause animatation if the user is rotating around
  fAnimationDelta := m_fElapsedTime;
  if (m_Camera.IsBeingDragged) then fAnimationDelta := 0.0;
  m_fAnimationTime := m_fAnimationTime + fAnimationDelta;

  // Animation the blobs
  pos := (1.0 + Cos(2 * D3DX_PI * m_fAnimationTime/3.0));
  m_BlobPoints[1].pos.x := pos;
  m_BlobPoints[2].pos.x := -pos;
  m_BlobPoints[3].pos.y := pos;
  m_BlobPoints[4].pos.y := -pos;

  FillBlobVB;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  i: LongWord;

  pSurfOldRenderTarget: IDirect3DSurface9;
  pSurfOldDepthStencil: IDirect3DSurface9;
  pGBufSurf: array[0..3] of IDirect3DSurface9;

  pSurfBlank: IDirect3DSurface9;
  
  iPass, nNumPasses: LongWord;
begin
  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 255), 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Get the initial device surfaces
    m_pd3dDevice.GetRenderTarget(0, pSurfOldRenderTarget);
    m_pd3dDevice.GetDepthStencilSurface(pSurfOldDepthStencil);

    // Turn off Z
    m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);
    m_pd3dDevice.SetDepthStencilSurface(nil);

    // Clear the blank texture
    m_pTexScratch.GetSurfaceLevel(0, pSurfBlank);
    m_pd3dDevice.SetRenderTarget(0, pSurfBlank);
    m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, D3DCOLOR_ARGB(0,0,0,0), 1.0, 0);
    SAFE_RELEASE(pSurfBlank);

    // clear temp textures
    for i:= 0 to 3 do
    begin
      m_pTexGBuffer[i].GetSurfaceLevel(0, pGBufSurf[i]);
      m_pd3dDevice.SetRenderTarget(0, pGBufSurf[i]);
      m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET, D3DCOLOR_ARGB(0,0,0,0), 1.0, 0);
    end;

    m_pd3dDevice.SetStreamSource(0, m_pBlobVB, 0, SizeOf(TScreenVertex));
    m_pd3dDevice.SetFVF(TScreenVertex_FVF);


    // Render the blobs
    m_pEffect.SetTechnique('BlobBlend');

    m_pEffect._Begin(@nNumPasses, 0);

    for iPass:= 0 to nNumPasses - 1 do
    begin
      m_pEffect.Pass(iPass);

      for i:= 0 to NUM_BLOBS - 1 do
      begin
        // Copy bits off of render target into scratch surface [for blending].
        m_pEffect.SetTexture('g_tSourceBlob', m_pTexScratch);
        m_pEffect.SetTexture('g_tSurfaceBuffer', m_pTexGBuffer[0]);
        m_pEffect.SetTexture('g_tMatrixBuffer', m_pTexGBuffer[1]);
        m_pd3dDevice.SetRenderTarget(0, pGBufSurf[2]);
        m_pd3dDevice.SetRenderTarget(1, pGBufSurf[3]);
        m_pd3dDevice.DrawPrimitive(D3DPT_TRIANGLELIST, i*6, 2);

        // Render the blob
        m_pEffect.SetTexture('g_tSourceBlob', m_pTexBlob);
        m_pEffect.SetTexture('g_tSurfaceBuffer', m_pTexGBuffer[2]);
        m_pEffect.SetTexture('g_tMatrixBuffer', m_pTexGBuffer[3]);
        m_pd3dDevice.SetRenderTarget(0, pGBufSurf[0]);
        m_pd3dDevice.SetRenderTarget(1, pGBufSurf[1]);
        m_pd3dDevice.DrawPrimitive(D3DPT_TRIANGLELIST, i*6, 2);
      end;
    end;

    m_pEffect._End;


    // Restore initial device surfaces
    m_pd3dDevice.SetDepthStencilSurface( pSurfOldDepthStencil);
    m_pd3dDevice.SetRenderTarget(0, pSurfOldRenderTarget);
    m_pd3dDevice.SetRenderTarget(1, nil);

    // Light and composite blobs into backbuffer
    m_pEffect.SetTechnique('BlobLight');

    m_pEffect._Begin(@nNumPasses, 0);

    for iPass:= 0 to nNumPasses - 1 do
    begin
      m_pEffect.Pass(iPass);

      for i:= 0 to NUM_BLOBS - 1 do
      begin
        m_pEffect.SetTexture('g_tSourceBlob', m_pTexGBuffer[0]);
        m_pEffect.SetTexture('g_tSurfaceBuffer', m_pTexGBuffer[1]);
        m_pEffect.SetTexture('g_tMatrixBuffer', m_pTexGBuffer[1]);
        m_pEffect.SetTexture('g_tEnvMap', m_pEnvMap);

        RenderFullScreenQuad(1.0);
      end;
    end;

    m_pEffect._End;

    // Render statistics
    RenderText;

    // End the scene
    m_pd3dDevice.EndScene;
  end;

{ // Not needed for Delphi
  SAFE_RELEASE(pSurfOldRenderTarget);
  SAFE_RELEASE(pSurfOldDepthStencil);

  for i:= 0 to 3 do SAFE_RELEASE(pGBufSurf[i]);
}
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderFullScreenQuad()
// Desc: Render a quad at the specified tranformed depth
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderFullScreenQuad(fDepth: Single): HRESULT;
var
  aVertices: array[0..3] of TScreenVertex;
  i: Integer;
begin
  aVertices[0].pos := D3DXVector4(-0.5, -0.5, fDepth, fDepth);
  aVertices[1].pos := D3DXVector4(-0.5, m_d3dsdBackBuffer.Height - 0.5, fDepth, fDepth);
  aVertices[2].pos := D3DXVector4(m_d3dsdBackBuffer.Width - 0.5, -0.5, fDepth, fDepth);
  aVertices[3].pos := D3DXVector4(m_d3dsdBackBuffer.Width - 0.5, m_d3dsdBackBuffer.Height - 0.5, fDepth, fDepth);

  aVertices[0].t0 := D3DXVector2(0.0, 0.0);
  aVertices[1].t0 := D3DXVector2(0.0, 1.0);
  aVertices[2].t0 := D3DXVector2(1.0, 0.0);
  aVertices[3].t0 := D3DXVector2(1.0, 1.0);

  for i:= 0 to 3 do
  with aVertices[i] do
  begin
    t2 := t0; t1 := t2;
    t1.x := t1.x + (1.0/m_d3dsdBackBuffer.Width);
    t1.y := t1.y + (1.0/m_d3dsdBackBuffer.Height);
  end;

  m_pd3dDevice.SetFVF(TScreenVertex_FVF);
  m_pd3dDevice.DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, aVertices, SizeOf(TScreenVertex));

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Render the device statistics and help information
//-----------------------------------------------------------------------------
procedure CMyD3DApplication.RenderText;
var
  rc: TRect;
  iLineY: Integer;
begin
  if (nil = m_pFont) then Exit;

{$IFDEF UNICODE}
  For UNICODE you should use "m_pFont.DrawTextW" routine!!!
{$ENDIF}

  // Output statistics
  iLineY := 0;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  m_pFont.DrawTextA(nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
  SetRect(rc, 2, iLineY, 0, 0); // Inc(iLineY, 15); //BAA: never used anyway
  m_pFont.DrawTextA( nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));

  // Draw help
  if m_bShowHelp then
  begin
    iLineY := m_d3dsdBackBuffer.Height-15*6;
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    m_pFont.DrawTextA(nil, 'Controls:', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));

    SetRect(rc, 20, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Rotate model: Left mouse button'#10 +
                           'Rotate camera: Right mouse button'#10 +
                           'Zoom camera: Mouse wheel scroll'#10 +
                           'Hide help: F1'#10,
                           -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));


    SetRect(rc, 250, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Change Device: F2'#10 +
                           'View readme: F9'#10 +
                           'Quit: ESC',
                           -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
  end else
  begin
    SetRect(rc, 2, 30, 0, 0);
    m_pFont.DrawTextA(nil, 'Press F1 for help',
                      -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
  end;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  if (dwBehavior and D3DCREATE_PUREDEVICE <> 0) then Exit;

  // Debugging vertex shaders requires either REF or software vertex processing
  // and debugging pixel shaders requires REF.
  {$IFDEF DEBUG_VS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) and
     ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) then Exit;
  {$ENDIF}

  // No fallback, so need ps2.0
  if (pCaps.PixelShaderVersion < D3DPS_VERSION(2,0)) then Exit;

  // No fallback, so need multiple render targets
  if (pCaps.NumSimultaneousRTs < 2) then Exit;

  // If device doesn't support 1.1 vertex shaders in HW, switch to SWVP.
  if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,1)) then
  begin
    if ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  end;

  // No fallback, so need to support render target
  if FAILED(m_pD3D.CheckDeviceFormat(pCaps.AdapterOrdinal, pCaps.DeviceType,
                                     adapterFormat, D3DUSAGE_RENDERTARGET,
                                     D3DRTYPE_TEXTURE, backBufferFormat))
  then Exit;

  Result:= S_OK;
end;

end.


